/*
 * Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package monasca.common.middleware;

/**
 * An exception to indicate any authentication error.
 * 
 * @author liemmn
 *
 */
public class AuthException extends RuntimeException {

  private static final long serialVersionUID = 5860956829821067827L;

  public AuthException(String msg) {
		super(msg);
	}
	
	public AuthException(String msg, Exception e) {
		super(msg, e);
	}
}
