/*
 * Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package monasca.common.util;

public final class Conversions {

  /**
   * Converts a Java Object of type Number to an Integer
   * @param variant Java Object of type Number
   * @return Integer
   * @exception IllegalArgumentException
   */
  public static Integer variantToInteger(Object variant) {
    if (variant instanceof Number) {
      return ((Number) variant).intValue();
    } else {
      throw new IllegalArgumentException(String.format("Variant of type \"%s\", and value \"%s\" is not a Number.",
                                                       variant.getClass(), variant));
    }
  }
}